{% extends "views/forms/base.htm" %}
{% load i18n %}

{% block form_controls %}
{% endblock form_controls %}

{% block form_content %}

<div class="row margin-bottom-10">
    <div class="col-xs-12">

        <!-- Spacer -->
        <div class="margin-top-20"></div>

        <div class="row">
            <div class="col-xs-1 text-right margin-top-10">
                <i class="color-red fa fa-3x fa-warning" style="margin-right: -25px; margin-top: 4px;"></i>
            </div>

            <div class="col-xs-11">
                <h6 class="warning">
                    {% trans "You won't be able to undo this operation!" %}&nbsp;  {% trans 'Are you sure you want to permanently delete this resource from Arches?' %}
                </h6>
            </div>
        </div>

        <!-- Spacer -->
        <div class="margin-top-20"></div>

        <div class="pull-right">
            <button class="btn btn-danger delete-resource-btn" type="button">{% trans 'Delete' %}</button>
        </div>

    </div>
</div>

{% endblock form_content %}