/*Wrapping element for Sticky-Footer*/
html {
  	min-height: 100%;
  	position: relative;
}
body {
  	/* Margin bottom by footer height */
  	margin-bottom: 60px;
}

/*404 Error Page V4
------------------------------------*/
.error-v4 {
	padding-top: 30px;
	text-align: center;	
}

.error-v4 img {
	margin-bottom: 20px;
}

.error-v4 h1 {
	color: #eee;
	font-size: 220px;
	line-height: 260px;
}

.error-v4 span.sorry {
	color: #fff;
	display: block;
	font-size: 25px;
	padding-bottom: 40px;
}

/*Sticky-Footer
------------------------------------*/
.sticky-footer {
  	position: absolute;
  	bottom: 0;
  	width: 100%;
  	padding: 20px;
  	/* Set the fixed height of the footer here */
  	height: 60px;
}

.sticky-footer .copyright-space {
	color: #eee;
	text-align: center;
}