

<!-- Service Blocks -->
<div class="row margin-bottom-40">
    <div class="col-sm-4">
        <div class="servive-block servive-block-default">
            <i class="icon-custom rounded-x icon-bg-dark fa fa-trophy"></i>
            <a class= "" href='?page=1&termFilter=%5B%7B"inverted"%3Afalse%2C"type"%3A"string"%2C"context"%3A"string"%2C"id"%3A"Red"%2C"text"%3A"Red"%2C"value"%3A"Red"%7D%5D&year_min_max=%5B%5D&temporalFilter=%5B%5D&spatialFilter=%7B"geometry"%3A%7B"type"%3A""%2C"coordinates"%3A%5B%5D%7D%2C"buffer"%3A%7B"width"%3A"0"%2C"unit"%3A"ft"%7D%7D&mapExpanded=true&timeExpanded=false'><h2 class="txt-color-blue">RED</h2></a>
            <p>Like the color red? Try it on for size!</p>                        
        </div>
    </div>

    <div class="col-sm-4">
        <div class="servive-block servive-block-default">            
            <i class="icon-custom rounded-x icon-bg-dark fa fa-bookmark-o"></i>
            <a class= "" href='?page=1&termFilter=%5B%5D&year_min_max=%5B%5D&temporalFilter=%5B%5D&spatialFilter=%7B"geometry"%3A%7B"type"%3A"LineString"%2C"coordinates"%3A%5B%5B-118.47105546545524%2C34.161227417492%5D%2C%5B-118.47036881994744%2C34.141339048545035%5D%2C%5B-118.4738020474865%2C34.12656180036532%5D%2C%5B-118.48135514807244%2C34.11519292840329%5D%2C%5B-118.47792192053338%2C34.09927393998204%5D%2C%5B-118.47586198400994%2C34.08392065037316%5D%2C%5B-118.45457597326775%2C34.05832564729579%5D%2C%5B-118.40376420568961%2C33.99458896676359%5D%2C%5B-118.39415116858032%2C33.979786106733556%5D%2C%5B-118.36943193029909%2C33.96668911993822%5D%2C%5B-118.3708052213147%2C33.89661448401003%5D%2C%5B-118.35913224768188%2C33.888634855021905%5D%2C%5B-118.34059281897096%2C33.86925264820758%5D%2C%5B-118.31518693518188%2C33.859559894049895%5D%2C%5B-118.28360124182252%2C33.859559894049895%5D%2C%5B-118.2499556119397%2C33.825341379759834%5D%2C%5B-118.20326371740842%2C33.826482217652796%5D%2C%5B-118.18335099768186%2C33.81393216368552%5D%2C%5B-118.16996141027958%2C33.81478790768536%5D%2C%5B-118.15382524084602%2C33.80680063077995%5D%2C%5B-118.14283891272099%2C33.80594480688411%5D%2C%5B-118.1376890714124%2C33.802806712700146%5D%2C%5B-118.11605973791633%2C33.802806712700146%5D%2C%5B-118.09202714514288%2C33.78483177474921%5D%2C%5B-118.087907272096%2C33.7739879240482%5D%5D%7D%2C"buffer"%3A%7B"width"%3A"10000"%2C"unit"%3A"ft"%7D%7D&mapExpanded=true&timeExpanded=false'><h2 class="txt-color-blue">I-405 Corrodor</h2></a>
            <p>What's next to the highway</p>
        </div>
    </div> 

    <div class="col-sm-4">
        <div class="servive-block servive-block-default">            
            <i class="icon-custom rounded-x icon-bg-dark fa fa-tasks"></i>
            <a class= "" href='?page=1&termFilter=%5B%7B"inverted"%3Afalse%2C"type"%3A"term"%2C"context"%3A"NAME.E41"%2C"id"%3A"Brady%20Bunch%20House%20Film%20Location"%2C"text"%3A"Brady%20Bunch%20House%20Film%20Location"%2C"value"%3A"Brady%20Bunch%20House%20Film%20Location"%7D%5D&year_min_max=%5B%5D&temporalFilter=%5B%5D&spatialFilter=%7B"geometry"%3A%7B"type"%3A""%2C"coordinates"%3A%5B%5D%7D%2C"buffer"%3A%7B"width"%3A"0"%2C"unit"%3A"ft"%7D%7D&mapExpanded=true&timeExpanded=false'><h2 class="txt-color-blue">The Brady Bunch!</h2></a>
            <p>Need I say more?</p>
        </div>
    </div>
</div>

<div class="row margin-bottom-40">
    <div class="col-sm-4">
        <div class="servive-block servive-block-default">
            <i class="icon-custom rounded-x icon-bg-dark fa fa-calendar-o"></i>
            <a class= "" href='?page=1&termFilter=%5B%5D&temporalFilter=%7B"year_min_max"%3A%5B1920%2C2013%5D%2C"filters"%3A%5B%7B"date"%3A"01%2F01%2F1930"%2C"date_types__value"%3A"82773828-583b-4b4b-8f50-625f11223deb"%2C"date_types__label"%3A"Built%20Date"%2C"date_operators__value"%3A"0"%2C"date_operators__label"%3A"Before"%7D%5D%2C"inverted"%3Afalse%7D&spatialFilter=%7B"geometry"%3A%7B"type"%3A"Point"%2C"coordinates"%3A%5B-118.40136094641244%2C34.07169289137546%5D%7D%2C"buffer"%3A%7B"width"%3A"50000"%2C"unit"%3A"ft"%7D%2C"inverted"%3Atrue%7D&mapExpanded=true&timeExpanded=true'><h2 class="txt-color-blue">Kitchen Sink</h2></a>
            <p>Everyting but...</p>
        </div>
    </div>

    <div class="col-sm-4">
        <div class="servive-block servive-block-default">
            <i class="icon-custom rounded-x icon-bg-dark fa fa-group"></i>
            <a class= "" href='?page=1&termFilter=%5B%7B"inverted"%3Afalse%2C"type"%3A"concept"%2C"context"%3A"Style"%2C"id"%3A"Modern%2C%20Mid-Century"%2C"text"%3A"Modern%2C%20Mid-Century"%2C"value"%3A"58fa926d-7652-4930-ab05-eebb4bd00f8b"%7D%5D&year_min_max=%5B%5D&temporalFilter=%5B%5D&spatialFilter=%7B"geometry"%3A%7B"type"%3A""%2C"coordinates"%3A%5B%5D%7D%2C"buffer"%3A%7B"width"%3A"50000"%2C"unit"%3A"ft"%7D%7D&mapExpanded=true&timeExpanded=true'><h2 class="txt-color-blue">Mid-Century Modern Homes</h2></a>
            <p></p>
        </div>
    </div>

    <div class="col-sm-4">
        <div class="servive-block servive-block-default">
            <i class="icon-custom rounded-x icon-bg-dark fa fa-file-text-o"></i>
            <a class= "" href=''><h2 class="txt-color-blue">Information Resources</h2></a>
            <p>Are images, videos, documents or other information carriers that describe Historic Resources </p>
        </div>
    </div>
</div>