{% load staticfiles %}
{% load i18n %}

<!--=== Footer v6 ===-->
<div id="footer-v6" class="footer-v6">
    <div class="footer">
        <div class="container">
            <div class="row">
                
                <!-- Default About Us.  Edit the <p> text below for your own message -->
                <div class="col-md-3 sm-margin-bottom-40">
                    <div class="heading-footer">
                        <h2>About Arches</h2>
                    </div>
                    <p>Arches is built to create and manage rich inventories of culturally significant resources.</p>
                </div>
                <!-- End About Us -->

                <!-- Recent News -->
                <div class="col-md-3 sm-margin-bottom-40">
                    <div class="heading-footer"><h2>Recent News</h2></div>
                    <ul class="list-unstyled link-news">
                        <li>
                            <a href="http://www.wmf.org/">WMF Introduces Arches v3.0</a>
                            <small>5 May, 2015</small>
                        </li>
                        <li>
                            <a href="#">Getty Announces HistoricPlacesLA</a>
                            <small>23 February, 2015</small>
                        </li>
                        <li>
                            <a href="http://www.archesproject.org">Arches-HIP Launched</a>
                            <small>20 April, 2015</small>
                        </li>
                    </ul>
                </div>
                <!-- End Recent News -->

                <!-- Useful Links -->
                <div class="col-md-3 sm-margin-bottom-40">
                    <div class="heading-footer"><h2>Useful Links</h2></div>
                    <ul class="list-unstyled footer-link-list">
                        <li><a href="{% url 'home' %}">Home</a></li>
                        <li><a href="">Search</a></li>
                        <li><a href="{% url 'map' %}">Maps</a></li>
                        <li><a href="#">Help</a></li>
                    </ul>
                </div>
                <!-- End Useful Links -->

                <!-- Contacts -->
                <div class="col-md-3">
                    <div class="heading-footer"><h2>Contacts</h2></div>
                    <ul class="list-unstyled contacts">
                        <li>
                            <i class="radius-3x fa fa-map-marker"></i>
                            601 Montgomery St, Suite 1095,<br>
                            San Francisco, CA 94111
                        </li>
                        <li>
                            <i class="radius-3x fa fa-phone"></i>
                            (+123) 456 7890 (main)<br>
                            (+123) 456 7891 (mobile)
                        </li>
                        <li>
                            <i class="radius-3x fa fa-globe"></i>
                            <a href="#">info@arches.org</a><br>                           
                            <a href="#">www.archesproject.org</a>
                        </li>
                    </ul>
                </div>
                <!-- End Contacts -->
            </div>
        </div><!--/container -->
    </div>
</div>
<!--=== End Footer v6 ===-->
